@@ -22,6 +22,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #include "gvcore.h"
 
 // Qt
+#include <QApplication>
 #include <QFuture>
 #include <QFutureWatcher>
 #include <QList>
@@ -41,6 +42,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #include <lib/document/documentfactory.h>
 #include <lib/gwenviewconfig.h>
 #include <lib/mimetypeutils.h>
+#include <lib/semanticinfo/semanticinfodirmodel.h>
+#include <lib/semanticinfo/sorteddirmodel.h>
 #include <lib/transformimageoperation.h>
 
 static const int MAX_RECENT_FOLDER = 20;
@@ -50,13 +53,15 @@ namespace Gwenview {
 
 struct GvCorePrivate {
 	QWidget* mParent;
+	SortedDirModel* mDirModel;
 };
 
 
-GvCore::GvCore(QWidget* parent)
+GvCore::GvCore(QWidget* parent, SortedDirModel* dirModel)
 : QObject(parent)
 , d(new GvCorePrivate) {
 	d->mParent = parent;
+	d->mDirModel = dirModel;
 }
 
 
@@ -92,37 +97,87 @@ void GvCore::addUrlToRecentFolders(const KUrl& _url) {
 }
 
 
-static void saveDocument(const KUrl& url) {
-	Document::Ptr doc = DocumentFactory::instance()->load(url);
-	doc->save(url, doc->format());
-}
+struct SaveAllHelper {
+	typedef QPair<KUrl, QString> ErrorListItem;
+	typedef QList<ErrorListItem> ErrorList;
 
+	SaveAllHelper(ErrorList* errorList)
+	: mErrorList(errorList) {}
+
+	void operator()(const KUrl& url) {
+		Document::Ptr doc = DocumentFactory::instance()->load(url);
+		if (!doc->save(url, doc->format())) {
+			*mErrorList << qMakePair(url, doc->errorString());
+		}
+	}
+
+	ErrorList* mErrorList;
+};
 
 void GvCore::saveAll() {
-	QList<KUrl> lst = DocumentFactory::instance()->modifiedDocumentList();
+	SaveAllHelper::ErrorList errorList;
+	SaveAllHelper helper(&errorList);
+
+	// Separate local and remote urls because saving to remote urls uses KIO
+	// methods, which are not thread-safe.
+	KUrl::List localUrls;
+	KUrl::List remoteUrls;
+	Q_FOREACH(const KUrl& url, DocumentFactory::instance()->modifiedDocumentList()) {
+		if (url.isLocalFile()) {
+			localUrls << url;
+		} else {
+			remoteUrls << url;
+		}
+	}
 
 	QProgressDialog progress(d->mParent);
 	progress.setLabelText(i18nc("@info:progress saving all image changes", "Saving..."));
 	progress.setCancelButtonText(i18n("&Stop"));
 	progress.setMinimum(0);
-	progress.setMaximum(lst.size());
+	progress.setMaximum(localUrls.size() + remoteUrls.size());
 	progress.setWindowModality(Qt::WindowModal);
+	progress.show();
 
-	QFuture<void> future = QtConcurrent::map(lst, saveDocument);
+	// Save local urls
+	QFuture<void> future = QtConcurrent::map(localUrls, helper);
 
 	QFutureWatcher<void> watcher;
 	watcher.setFuture(future);
 	connect(&watcher, SIGNAL(progressValueChanged(int)),
 		&progress, SLOT(setValue(int)) );
 
-	connect(&watcher, SIGNAL(finished()),
-		&progress, SLOT(close()) );
-
 	connect(&progress, SIGNAL(canceled()),
 		&watcher, SLOT(cancel()) );
 
-	progress.exec();
 	watcher.waitForFinished();
+
+	// Save remote urls
+	Q_FOREACH(const KUrl& url, remoteUrls) {
+		if (progress.wasCanceled()) {
+			break;
+		}
+		helper(url);
+		progress.setValue(progress.value() + 1);
+		qApp->processEvents();
+	}
+
+	progress.close();
+
+	// Done, show message if necessary
+	if (errorList.count() > 0) {
+		QString msg = i18ncp("@info", "One document could not be saved:", "%1 documents could not be saved:", errorList.count());
+		msg += "<ul>";
+		Q_FOREACH(const SaveAllHelper::ErrorListItem& item, errorList) {
+			const KUrl& url = item.first;
+			QString name = url.fileName().isEmpty() ? url.pathOrUrl() : url.fileName();
+			msg += "<li>"
+				+ i18nc("@info %1 is the name of the document which failed to save, %2 is the reason for the failure",
+					"<filename>%1</filename>: %2", name, item.second)
+				+ "</li>";
+		}
+		msg += "</ul>";
+		KMessageBox::sorry(d->mParent, msg);
+	}
 }
 
 
@@ -133,8 +188,12 @@ void GvCore::save(const KUrl& url) {
 	QByteArray format = doc->format();
 	QStringList availableTypes = KImageIO::types(KImageIO::Writing);
 	if (availableTypes.contains(QString(format))) {
-		doc->save(url, format);
-		// FIXME: check return code of Document::save()
+		if (!doc->save(url, format)) {
+			QString name = url.fileName().isEmpty() ? url.pathOrUrl() : url.fileName();
+			QString msg = i18nc("@info", "<b>Saving <filename>%1</filename> failed:</b><br>%2",
+				name, doc->errorString());
+			KMessageBox::sorry(d->mParent, msg);
+		}
 		return;
 	}
 
@@ -157,18 +216,40 @@ void GvCore::save(const KUrl& url) {
 void GvCore::saveAs(const KUrl& url) {
 	KFileDialog dialog(url, QString(), d->mParent);
 	dialog.setOperationMode(KFileDialog::Saving);
-
-	// Init mime filter
-	QString mimeType = MimeTypeUtils::urlMimeType(url);
-	QStringList availableMimeTypes = KImageIO::mimeTypes(KImageIO::Writing);
-	dialog.setMimeFilter(availableMimeTypes, mimeType);
+	dialog.setMimeFilter(
+		KImageIO::mimeTypes(KImageIO::Writing), // List
+		MimeTypeUtils::urlMimeType(url)         // Default
+		);
 
 	// Show dialog
-	if (!dialog.exec()) {
-		return;
-	}
+	QByteArray format;
+	do {
+		if (!dialog.exec()) {
+			return;
+		}
 
-	KUrl saveAsUrl = dialog.selectedUrl();
+		const QString mimeType = dialog.currentMimeFilter();
+		if (mimeType.isEmpty()) {
+			KMessageBox::sorry(
+				d->mParent,
+				i18nc("@info",
+					"No image format selected.")
+				);
+			continue;
+		}
+
+		const QStringList typeList = KImageIO::typeForMime(mimeType);
+		if (typeList.count() > 0) {
+			format = typeList[0].toAscii();
+			break;
+		}
+		KMessageBox::sorry(
+			d->mParent,
+			i18nc("@info",
+				"Gwenview cannot save images as %1.", mimeType)
+			);
+	} while (true);
+	const KUrl saveAsUrl = dialog.selectedUrl();
 
 	// Check for overwrite
 	if (KIO::NetAccess::exists(saveAsUrl, KIO::NetAccess::DestinationSide, d->mParent)) {
@@ -186,31 +267,55 @@ void GvCore::saveAs(const KUrl& url) {
 	}
 
 	// Start save
-	mimeType = dialog.currentMimeFilter();
-	QStringList typeList = KImageIO::typeForMime(mimeType);
-	Q_ASSERT(typeList.count() > 0);
 	Document::Ptr doc = DocumentFactory::instance()->load(url);
-	doc->save(saveAsUrl, typeList[0].toAscii());
+	doc->save(saveAsUrl, format.data());
+}
+
+
+static void applyTransform(const KUrl& url, Orientation orientation) {
+	if (!GvCore::ensureDocumentIsEditable(url)) {
+		return;
+	}
+	TransformImageOperation* op = new TransformImageOperation(orientation);
+	Document::Ptr doc = DocumentFactory::instance()->load(url);
+	op->setDocument(doc);
+	doc->undoStack()->push(op);
 }
 
 
 void GvCore::rotateLeft(const KUrl& url) {
-	TransformImageOperation* op = new TransformImageOperation(ROT_270);
-	Document::Ptr doc = DocumentFactory::instance()->load(url);
-	doc->loadFullImage();
-	doc->waitUntilLoaded();
-	op->setDocument(doc);
-	doc->undoStack()->push(op);
+	applyTransform(url, ROT_270);
 }
 
 
 void GvCore::rotateRight(const KUrl& url) {
-	TransformImageOperation* op = new TransformImageOperation(ROT_90);
+	applyTransform(url, ROT_90);
+}
+
+
+void GvCore::setRating(const KUrl& url, int rating) {
+	QModelIndex index = d->mDirModel->indexForUrl(url);
+	if (!index.isValid()) {
+		kWarning() << "invalid index!";
+		return;
+	}
+	d->mDirModel->setData(index, rating, SemanticInfoDirModel::RatingRole);
+}
+
+
+bool GvCore::ensureDocumentIsEditable(const KUrl& url) {
 	Document::Ptr doc = DocumentFactory::instance()->load(url);
 	doc->loadFullImage();
 	doc->waitUntilLoaded();
-	op->setDocument(doc);
-	doc->undoStack()->push(op);
+	if (doc->isEditable()) {
+		return true;
+	}
+
+	KMessageBox::sorry(
+		QApplication::activeWindow(),
+		i18nc("@info", "Gwenview cannot edit this kind of image.")
+		);
+	return false;
 }
 
 
