@@ -35,6 +35,15 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 namespace Gwenview {
 
+#undef ENABLE_LOG
+#undef LOG
+//#define ENABLE_LOG
+#ifdef ENABLE_LOG
+#define LOG(x) kDebug() << x
+#else
+#define LOG(x) ;
+#endif
+
 
 struct ImageViewPrivate {
 	ImageView* mView;
@@ -50,6 +59,7 @@ struct ImageViewPrivate {
 	QPixmap mAlternateBuffer;
 	ImageScaler* mScaler;
 	QPointer<AbstractImageViewTool> mTool;
+	QPointer<AbstractImageViewTool> mDefaultTool;
 	bool mInsideSetZoom;
 
 
@@ -167,6 +177,13 @@ struct ImageViewPrivate {
 	}
 
 
+	void startAnimationIfNecessary() {
+		if (mDocument && mView->isVisible()) {
+			mDocument->startAnimation();
+		}
+	}
+
+
 	// At least gcc 3.4.6 on FreeBSD requires a default constructor.
 	ImageViewPrivate() { }
 };
@@ -228,6 +245,7 @@ void ImageView::setEnlargeSmallerImages(bool value) {
 
 void ImageView::setDocument(Document::Ptr document) {
 	if (d->mDocument) {
+		d->mDocument->stopAnimation();
 		disconnect(d->mDocument.data(), 0, this, 0);
 	}
 	d->mDocument = document;
@@ -236,20 +254,23 @@ void ImageView::setDocument(Document::Ptr document) {
 		return;
 	}
 
-	connect(d->mDocument.data(), SIGNAL(metaDataLoaded(const KUrl&)),
-		SLOT(slotDocumentMetaDataLoaded()) );
+	connect(d->mDocument.data(), SIGNAL(metaInfoLoaded(const KUrl&)),
+		SLOT(slotDocumentMetaInfoLoaded()) );
+	connect(d->mDocument.data(), SIGNAL(isAnimatedUpdated()),
+		SLOT(slotDocumentIsAnimatedUpdated()) );
 
-	if (d->mDocument->size().isValid()) {
-		finishSetDocument();
+	const Document::LoadingState state = d->mDocument->loadingState();
+	if (state == Document::MetaInfoLoaded || state == Document::Loaded) {
+		slotDocumentMetaInfoLoaded();
 	}
 }
 
 
-void ImageView::slotDocumentMetaDataLoaded() {
+void ImageView::slotDocumentMetaInfoLoaded() {
 	if (d->mDocument->size().isValid()) {
 		finishSetDocument();
 	} else {
-		// Could not retrieve image size from meta data, we need to load the
+		// Could not retrieve image size from meta info, we need to load the
 		// full image now.
 		connect(d->mDocument.data(), SIGNAL(loaded(const KUrl&)),
 			SLOT(finishSetDocument()) );
@@ -280,6 +301,8 @@ void ImageView::finishSetDocument() {
 		updateImageRect(rect);
 		updateScrollBars();
 	}
+
+	d->startAnimationIfNecessary();
 	d->mViewport->update();
 }
 
@@ -289,7 +312,13 @@ Document::Ptr ImageView::document() const {
 }
 
 
+void ImageView::slotDocumentIsAnimatedUpdated() {
+	d->startAnimationIfNecessary();
+}
+
+
 void ImageView::updateImageRect(const QRect& imageRect) {
+	LOG("imageRect" << imageRect);
 	QRect viewportRect = mapToViewport(imageRect);
 	viewportRect = viewportRect.intersected(d->mViewport->rect());
 	if (viewportRect.isEmpty()) {
@@ -298,6 +327,7 @@ void ImageView::updateImageRect(const QRect& imageRect) {
 
 	QSize bufferSize = d->requiredBufferSize();
 	if (bufferSize != d->mCurrentBuffer.size()) {
+		LOG("Need a new buffer");
 		// Since the required buffer size is not the same as our current buffer
 		// size, the image must have been resized. Call setDocument(), it will
 		// take care of resizing the buffer and repainting the whole image.
@@ -356,13 +386,8 @@ QPoint ImageView::imageOffset() const {
 	return QPoint(left, top);
 }
 
-void ImageView::setZoom(qreal zoom) {
-	QPoint center = QPoint(d->mViewport->width() / 2, d->mViewport->height() / 2);
-	setZoom(zoom, center);
-}
 
-
-void ImageView::setZoom(qreal zoom, const QPoint& center) {
+void ImageView::setZoom(qreal zoom, const QPoint& _center) {
 	if (!d->mDocument) {
 		return;
 	}
@@ -373,6 +398,13 @@ void ImageView::setZoom(qreal zoom, const QPoint& center) {
 	}
 	d->mZoom = zoom;
 
+	QPoint center;
+	if (_center == QPoint(-1, -1)) {
+		center = QPoint(d->mViewport->width() / 2, d->mViewport->height() / 2);
+	} else {
+		center = _center;
+	}
+
 	// If we zoom more than twice, then assume the user wants to see the real
 	// pixels, for example to fine tune a crop operation
 	if (d->mZoom < 2.) {
@@ -434,7 +466,7 @@ void ImageView::setZoom(qreal zoom, const QPoint& center) {
 
 	d->mScaler->setZoom(d->mZoom);
 	d->setScalerRegionToVisibleRect();
-	emit zoomChanged();
+	emit zoomChanged(d->mZoom);
 }
 
 qreal ImageView::zoom() const {
@@ -519,6 +551,7 @@ void ImageView::scrollContentsBy(int dx, int dy) {
 
 
 void ImageView::updateFromScaler(int zoomedImageLeft, int zoomedImageTop, const QImage& image) {
+	LOG("");
 	int viewportLeft = zoomedImageLeft - d->hScroll();
 	int viewportTop = zoomedImageTop - d->vScroll();
 
@@ -554,7 +587,7 @@ void ImageView::setCurrentTool(AbstractImageViewTool* tool) {
 	if (d->mTool) {
 		d->mTool->toolDeactivated();
 	}
-	d->mTool = tool;
+	d->mTool = tool ? QPointer<AbstractImageViewTool>(tool) : d->mDefaultTool;
 	if (d->mTool) {
 		d->mTool->toolActivated();
 	}
@@ -567,7 +600,20 @@ AbstractImageViewTool* ImageView::currentTool() const {
 }
 
 
-QPoint ImageView::mapToViewport(const QPoint& src) {
+void ImageView::setDefaultTool(AbstractImageViewTool* tool) {
+	d->mDefaultTool = tool;
+	if (!d->mTool && tool) {
+		setCurrentTool(tool);
+	}
+}
+
+
+AbstractImageViewTool* ImageView::defaultTool() const {
+	return d->mDefaultTool;
+}
+
+
+QPoint ImageView::mapToViewport(const QPoint& src) const {
 	QPoint dst(int(src.x() * d->mZoom), int(src.y() * d->mZoom));
 
 	dst += imageOffset();
@@ -579,7 +625,19 @@ QPoint ImageView::mapToViewport(const QPoint& src) {
 }
 
 
-QRect ImageView::mapToViewport(const QRect& src) {
+QPointF ImageView::mapToViewportF(const QPointF& src) const {
+	QPointF dst(src.x() * d->mZoom, src.y() * d->mZoom);
+
+	dst += imageOffset();
+
+	dst.rx() -= d->hScroll();
+	dst.ry() -= d->vScroll();
+
+	return dst;
+}
+
+
+QRect ImageView::mapToViewport(const QRect& src) const {
 	QRect dst(
 		mapToViewport(src.topLeft()),
 		mapToViewport(src.bottomRight())
@@ -588,7 +646,16 @@ QRect ImageView::mapToViewport(const QRect& src) {
 }
 
 
-QPoint ImageView::mapToImage(const QPoint& src) {
+QRectF ImageView::mapToViewportF(const QRectF& src) const {
+	QRectF dst(
+		mapToViewportF(src.topLeft()),
+		mapToViewportF(src.bottomRight())
+	);
+	return dst;
+}
+
+
+QPoint ImageView::mapToImage(const QPoint& src) const {
 	QPoint dst = src;
 	
 	dst.rx() += d->hScroll();
@@ -600,7 +667,19 @@ QPoint ImageView::mapToImage(const QPoint& src) {
 }
 
 
-QRect ImageView::mapToImage(const QRect& src) {
+QPointF ImageView::mapToImageF(const QPointF& src) const {
+	QPointF dst = src;
+
+	dst.rx() += d->hScroll();
+	dst.ry() += d->vScroll();
+
+	dst -= imageOffset();
+
+	return dst / d->mZoom;
+}
+
+
+QRect ImageView::mapToImage(const QRect& src) const {
 	QRect dst(
 		mapToImage(src.topLeft()),
 		mapToImage(src.bottomRight())
@@ -609,16 +688,17 @@ QRect ImageView::mapToImage(const QRect& src) {
 }
 
 
+QRectF ImageView::mapToImageF(const QRectF& src) const {
+	QRectF dst(
+		mapToImageF(src.topLeft()),
+		mapToImageF(src.bottomRight())
+	);
+	return dst;
+}
+
+
 qreal ImageView::computeZoomToFit() const {
-	if (!d->mDocument || !d->mDocument->size().isValid()) {
-		return 0;
-	}
-	int width = d->mViewport->width();
-	int height = d->mViewport->height();
-	qreal zoom = qreal(width) / d->mDocument->width();
-	if ( int(d->mDocument->height() * zoom) > height) {
-		zoom = qreal(height) / d->mDocument->height();
-	}
+	qreal zoom = qMin(computeZoomToFitWidth(), computeZoomToFitHeight());
 
 	if (!d->mEnlargeSmallerImages) {
 		zoom = qMin(zoom, qreal(1.0));
@@ -629,7 +709,7 @@ qreal ImageView::computeZoomToFit() const {
 
 
 qreal ImageView::computeZoomToFitWidth() const {
-	if (!d->mDocument) {
+	if (!d->mDocument || !d->mDocument->size().isValid()) {
 		return 1.;
 	}
 	return qreal(d->mViewport->width()) / d->mDocument->width();
@@ -637,13 +717,27 @@ qreal ImageView::computeZoomToFitWidth() const {
 
 
 qreal ImageView::computeZoomToFitHeight() const {
-	if (!d->mDocument) {
+	if (!d->mDocument || !d->mDocument->size().isValid()) {
 		return 1.;
 	}
 	return qreal(d->mViewport->height()) / d->mDocument->height();
 }
 
 
+void ImageView::showEvent(QShowEvent* event) {
+	QAbstractScrollArea::showEvent(event);
+	d->startAnimationIfNecessary();
+}
+
+
+void ImageView::hideEvent(QHideEvent* event) {
+	QAbstractScrollArea::hideEvent(event);
+	if (d->mDocument) {
+		d->mDocument->stopAnimation();
+	}
+}
+
+
 void ImageView::mousePressEvent(QMouseEvent* event) {
 	if (d->mTool) {
 		d->mTool->mousePressEvent(event);
