@@ -21,19 +21,22 @@
 
 */
 #include <qlayout.h>
-#include <qwidgetstack.h>
-#include <qvbuttongroup.h>
-#include <qvgroupbox.h>
+#include <q3widgetstack.h>
+
 #include <qcombobox.h>
 #include <qlineedit.h>
 #include <qradiobutton.h>
-#include <qwhatsthis.h>
-#include <qlabel.h>
-#include <qgrid.h>
+#include <Q3Button>
 
+#include <qlabel.h>
+#include <q3grid.h>
+//Added by qt3to4:
+#include <QHBoxLayout>
+#include <QVBoxLayout>
+#include <Q3VButtonGroup>
 #include <klocale.h>
 #include <kconfig.h>
-#include <klistview.h>
+#include <k3listview.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 
@@ -72,7 +75,7 @@ KCamera::~KCamera()
 
 bool KCamera::initInformation()
 {
-	if (!m_model)
+	if (m_model.isNull())
 		return false;
 
 	if(gp_abilities_list_new(&m_abilitylist) != GP_OK) {
@@ -83,10 +86,10 @@ bool KCamera::initInformation()
 		emit error(i18n("Could not load ability list."));
 		return false;
 	}
-	int index = gp_abilities_list_lookup_model(m_abilitylist, m_model.local8Bit().data());
+	int index = gp_abilities_list_lookup_model(m_abilitylist, m_model.toLocal8Bit().data());
 	if(index < 0) {
 		emit error(i18n("Description of abilities for camera %1 is not available."
-					" Configuration options may be incorrect.").arg(m_model));
+					" Configuration options may be incorrect.", m_model));
 		return false;
 	}
         gp_abilities_list_get_abilities(m_abilitylist, index, &m_abilities);
@@ -102,7 +105,7 @@ bool KCamera::initCamera()
 
 		initInformation();
 
-		if (!m_model || !m_path)
+		if (m_model.isNull() || m_path.isNull())
 			return false;
 
 		result = gp_camera_new(&m_camera);
@@ -117,12 +120,12 @@ bool KCamera::initCamera()
 		GPPortInfoList *il;
 		gp_port_info_list_new(&il);
 		gp_port_info_list_load(il);
-		gp_port_info_list_get_info(il, gp_port_info_list_lookup_path(il, m_path.local8Bit().data()), &info);
+		gp_port_info_list_get_info(il, gp_port_info_list_lookup_path(il, m_path.toLocal8Bit().data()), &info);
 		gp_port_info_list_free(il);
 		gp_camera_set_abilities(m_camera, m_abilities);
 		gp_camera_set_port_info(m_camera, info);
 
-		// this might take some time (esp. for non-existant camera) - better be done asynchronously
+		// this might take some time (esp. for non-existent camera) - better be done asynchronously
 		result = gp_camera_init(m_camera, glob_context);
 		if (result != GP_OK) {
 			gp_camera_free(m_camera);
@@ -192,24 +195,24 @@ bool KCamera::test()
 
 void KCamera::load(KConfig *config)
 {
-	config->setGroup(m_name);
+	KConfigGroup group = config->group(m_name);
 	if (m_model.isNull())
-		m_model = config->readEntry("Model");
+		m_model = group.readEntry("Model");
 	if (m_path.isNull())
-		m_path = config->readEntry("Path");
+		m_path = group.readEntry("Path");
 	invalidateCamera();
 }
 
 void KCamera::save(KConfig *config)
 {
-	config->setGroup(m_name);
-	config->writeEntry("Model", m_model);
-	config->writeEntry("Path", m_path);
+	KConfigGroup group = config->group(m_name);
+	group.writeEntry("Model", m_model);
+	group.writeEntry("Path", m_path);
 }
 
 QString KCamera::portName()
 {
-	QString port = m_path.left(m_path.find(":")).lower();
+	QString port = m_path.left(m_path.indexOf(":")).toLower();
 	if (port == "serial") return i18n("Serial");
 	if (port == "usb") return i18n("USB");
 	return i18n("Unknown port");
@@ -271,8 +274,13 @@ CameraAbilities KCamera::abilities()
 // ---------- KameraSelectCamera ------------
 
 KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *device)
-	: KDialogBase(parent, "kkameradeviceselect", true, i18n("Select Camera Device"), Ok | Cancel, Ok, true)
+	: KDialog(parent)
 {
+    setCaption( i18n("Select Camera Device") );
+    setButtons( Ok | Cancel );
+    setDefaultButton( Ok );
+    setModal( true );
+    showButtonSeparator( true );
 	m_device = device;
 	connect(m_device, SIGNAL(error(const QString &)),
 		SLOT(slot_error(const QString &)));
@@ -283,36 +291,40 @@ KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *dev
 	setMainWidget(page);
 
 	// a layout with vertical boxes
-	QHBoxLayout *topLayout = new QHBoxLayout(page, 0, KDialog::spacingHint());
+	QHBoxLayout *topLayout = new QHBoxLayout(page);
+	topLayout->setSpacing(KDialog::spacingHint());
+	topLayout->setMargin(0);
 
 	// the models list
-	m_modelSel = new KListView(page);
+	m_modelSel = new K3ListView(page);
 	topLayout->addWidget( m_modelSel );
 	m_modelSel->addColumn(i18n("Supported Cameras"));
-	m_modelSel->setColumnWidthMode(0, QListView::Maximum);
-	connect(m_modelSel, SIGNAL(selectionChanged(QListViewItem *)),
-        SLOT(slot_setModel(QListViewItem *)));
+	m_modelSel->setColumnWidthMode(0, Q3ListView::Maximum);
+	connect(m_modelSel, SIGNAL(selectionChanged(Q3ListViewItem *)),
+        SLOT(slot_setModel(Q3ListViewItem *)));
 	// make sure listview only as wide as it needs to be
 	m_modelSel->setSizePolicy(QSizePolicy(QSizePolicy::Maximum,
 		QSizePolicy::Preferred));
 
-	QVBoxLayout *rightLayout = new QVBoxLayout(0L, 0, KDialog::spacingHint());
+	QVBoxLayout *rightLayout = new QVBoxLayout();
+	rightLayout->setSpacing(KDialog::spacingHint());
+	rightLayout->setMargin(0);
 	topLayout->addLayout( rightLayout );
 
-	m_portSelectGroup = new QVButtonGroup(i18n("Port"), page);
+	m_portSelectGroup = new Q3VButtonGroup(i18n("Port"), page);
 	rightLayout->addWidget(m_portSelectGroup);
-	m_portSettingsGroup = new QVGroupBox(i18n("Port Settings"), page);
+	m_portSettingsGroup = new Q3GroupBox(1, Qt::Horizontal,i18n("Port Settings"), page);
 	rightLayout->addWidget(m_portSettingsGroup);
 
 	// Create port type selection radiobuttons.
 	m_serialRB = new QRadioButton(i18n("Serial"), m_portSelectGroup);
 	m_portSelectGroup->insert(m_serialRB, INDEX_SERIAL);
-	QWhatsThis::add(m_serialRB, i18n("If this option is checked, the camera would have to be connected one of the serial ports (known as COM in Microsoft Windows) in your computer."));
+	m_serialRB->setWhatsThis( i18n("If this option is checked, the camera would have to be connected one of the serial ports (known as COM in Microsoft Windows) in your computer."));
 	m_USBRB = new QRadioButton(i18n("USB"), m_portSelectGroup);
 	m_portSelectGroup->insert(m_USBRB, INDEX_USB);
-	QWhatsThis::add(m_USBRB, i18n("If this option is checked, the camera would have to be connected to one of the USB slots in your computer or USB hub."));
+	m_USBRB->setWhatsThis( i18n("If this option is checked, the camera would have to be connected to one of the USB slots in your computer or USB hub."));
 	// Create port settings widget stack
-	m_settingsStack = new QWidgetStack(m_portSettingsGroup);
+	m_settingsStack = new Q3WidgetStack(m_portSettingsGroup);
 	connect(m_portSelectGroup, SIGNAL(clicked(int)),
 		m_settingsStack, SLOT(raiseWidget(int)));
 
@@ -321,14 +333,15 @@ KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *dev
 		m_settingsStack), INDEX_NONE);
 
 	// serial tab
-	QGrid *grid = new QGrid(2, m_settingsStack);
+	Q3Grid *grid = new Q3Grid(2, m_settingsStack);
 	grid->setSpacing(KDialog::spacingHint());
 	new QLabel(i18n("Port:"), grid);
-	m_serialPortCombo = new QComboBox(TRUE, grid);
-	QWhatsThis::add(m_serialPortCombo, i18n("Here you should choose the serial port you connect the camera to."));
+	m_serialPortCombo = new QComboBox(grid);
+	m_serialPortCombo->setEditable(true);
+	m_serialPortCombo->setWhatsThis( i18n("Here you should choose the serial port you connect the camera to."));
 	m_settingsStack->addWidget(grid, INDEX_SERIAL);
 
-	grid = new QGrid(2, m_settingsStack);
+	grid = new Q3Grid(2, m_settingsStack);
 	grid->setSpacing(KDialog::spacingHint());
 	new QLabel(i18n("Port"), grid);
 
@@ -347,7 +360,7 @@ KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *dev
 	for (int i = 0; i < gphoto_ports; i++) {
 		if (gp_port_info_list_get_info(list, i, &info) >= 0) {
 			if (strncmp(info.path, "serial:", 7) == 0)
-				m_serialPortCombo->insertItem(QString::fromLatin1(info.path).mid(7));
+				m_serialPortCombo->addItem(QString::fromLatin1(info.path).mid(7));
 		}
 	}
 	gp_port_info_list_free(list);
@@ -358,7 +371,7 @@ KameraDeviceSelectDialog::KameraDeviceSelectDialog(QWidget *parent, KCamera *dev
 	populateCameraListView();
 	load();
 
-	enableButtonOK(false );
+	enableButtonOk(false );
     m_portSelectGroup->setEnabled( false );
     m_portSettingsGroup->setEnabled( false );
 }
@@ -376,7 +389,7 @@ bool KameraDeviceSelectDialog::populateCameraListView()
 	} else {
 		for(int x = 0; x < numCams; ++x) {
 			if(gp_abilities_list_get_abilities(m_device->m_abilitylist, x, &a) == GP_OK) {
-				new QListViewItem(m_modelSel, a.model);
+				new Q3ListViewItem(m_modelSel, a.model);
 			}
 		}
 		return true;
@@ -403,36 +416,34 @@ void KameraDeviceSelectDialog::save()
 void KameraDeviceSelectDialog::load()
 {
 	QString path = m_device->path();
-	QString port = path.left(path.find(":")).lower();
+	QString port = path.left(path.indexOf(":")).toLower();
 
 	if (port == "serial") setPortType(INDEX_SERIAL);
 	if (port == "usb") setPortType(INDEX_USB);
 
-	QListViewItem *modelItem = m_modelSel->firstChild();
-	if( modelItem)
-	{
-	do {
-		if (modelItem->text(0) == m_device->model()) {
-			m_modelSel->setSelected(modelItem, true);
-			m_modelSel->ensureItemVisible(modelItem);
-		}
-	} while ( ( modelItem = modelItem->nextSibling() ) );
-	}
+	Q3ListViewItem *modelItem = m_modelSel->firstChild();
+	if( modelItem )
+		do {
+			if (modelItem->text(0) == m_device->model()) {
+				m_modelSel->setSelected(modelItem, true);
+				m_modelSel->ensureItemVisible(modelItem);
+			}
+		} while ( ( modelItem = modelItem->nextSibling() ) );
 }
 
-void KameraDeviceSelectDialog::slot_setModel(QListViewItem *item)
+void KameraDeviceSelectDialog::slot_setModel(Q3ListViewItem *item)
 {
-    enableButtonOK(true);
+    enableButtonOk(true);
     m_portSelectGroup->setEnabled(true);
     m_portSettingsGroup->setEnabled(true);
 
     QString model = item->text(0);
 
 	CameraAbilities abilities;
-	int index = gp_abilities_list_lookup_model(m_device->m_abilitylist, model.local8Bit().data());
+	int index = gp_abilities_list_lookup_model(m_device->m_abilitylist, model.toLocal8Bit().data());
 	if(index < 0) {
 		slot_error(i18n("Description of abilities for camera %1 is not available."
-				" Configuration options may be incorrect.").arg(model));
+				" Configuration options may be incorrect.", model));
 	}
 	int result = gp_abilities_list_get_abilities(m_device->m_abilitylist, index, &abilities);
 	if (result == GP_OK) {
@@ -441,7 +452,7 @@ void KameraDeviceSelectDialog::slot_setModel(QListViewItem *item)
 		m_USBRB->setEnabled(abilities.port & GP_PORT_USB);
 
 		// turn off any selected port
-		QButton *selected = m_portSelectGroup->selected();
+		QAbstractButton *selected = m_portSelectGroup->selected();
 		if(selected != NULL)
 			selected->toggle();
 
@@ -452,7 +463,7 @@ void KameraDeviceSelectDialog::slot_setModel(QListViewItem *item)
 			setPortType(INDEX_USB);
 	} else {
 		slot_error(i18n("Description of abilities for camera %1 is not available."
-			     " Configuration options may be incorrect.").arg(model));
+			     " Configuration options may be incorrect.", model));
 	}
 }
 
