@@ -18,22 +18,23 @@
 #define KGAMMA_H_
 
 #include <kcmodule.h>
+//Added by qt3to4:
+#include <QList>
 
 class GammaCtrl;
 class QCheckBox;
 class QComboBox;
 class XVidExtWrap;
-class KProcess;
+class K3Process;
 
 class KGamma: public KCModule
 {
    Q_OBJECT
    public:
-      KGamma(QWidget *parent, const char *name, const QStringList&);
+      KGamma(QWidget* parent_P, const QVariantList &args );
       virtual ~KGamma();
 
       void load();
-      void load(bool useDefaults);
       void save();
       void defaults();
       int buttons();
@@ -65,12 +66,12 @@ class KGamma: public KCModule
       bool saved, GammaCorrection;
       int ScreenCount, currentScreen;
       QStringList rgamma, ggamma, bgamma;
-      QValueList<int> assign;
-      QValueList<float> rbak, gbak, bbak;
+      QList<int> assign;
+      QList<float> rbak, gbak, bbak;
       GammaCtrl *gctrl, *rgctrl, *ggctrl, *bgctrl;
       QCheckBox *xf86cfgbox, *syncbox;
       QComboBox *screenselect;
-      KProcess *rootProcess;
+      K3Process *rootProcess;
       XVidExtWrap *xv;
 };
 
