@@ -9,10 +9,6 @@
  * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
  * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
- * NOTE: Do not use kdDebug() in this implementation because 
- *       it will be multithreaded. Use qDebug() instead. 
- *       See B.K.O #133026 for details.
- *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
@@ -25,6 +21,14 @@
  *
  * ============================================================ */
 
+// C ANSI includes.
+
+extern "C"
+{
+#include <sys/stat.h>
+#include <utime.h>
+}
+
 // Local includes.
 
 #include "version.h"
@@ -83,6 +87,25 @@ KExiv2& KExiv2::operator=(const KExiv2& metadata)
 
 //-- Statics methods ----------------------------------------------
 
+bool KExiv2::initializeExiv2()
+{
+#ifdef _XMP_SUPPORT_
+    return (Exiv2::XmpParser::initialize());
+#endif // _XMP_SUPPORT_
+
+    return true;
+}
+
+bool KExiv2::cleanupExiv2()
+{
+    // Fix memory leak if Exiv2 support XMP.
+#ifdef _XMP_SUPPORT_
+    Exiv2::XmpParser::terminate();
+#endif // _XMP_SUPPORT_
+
+    return true;
+}
+
 bool KExiv2::supportXmp()
 {
 #ifdef _XMP_SUPPORT_
@@ -92,13 +115,46 @@ bool KExiv2::supportXmp()
 #endif // _XMP_SUPPORT_
 }
 
-bool KExiv2::supportTiffWritting()
+bool KExiv2::supportMetadataWritting(const QString& typeMime)
 {
+    if (typeMime == QString("image/jpeg"))
+    {
+        return true;
+    }
+    else if (typeMime == QString("image/tiff"))
+    {
 #if (EXIV2_TEST_VERSION(0,17,91))
-    return true;
+        return true;
 #else
-    return false;
+        return false;
 #endif
+    }
+    else if (typeMime == QString("image/png"))
+    {
+#if (EXIV2_TEST_VERSION(0,17,91))
+        return true;
+#else
+        return false;
+#endif
+    }
+    else if (typeMime == QString("image/jp2"))
+    {
+#if (EXIV2_TEST_VERSION(0,17,91))
+        return true;
+#else
+        return false;
+#endif
+    }
+    else if (typeMime == QString("image/x-raw"))
+    {
+#if (EXIV2_TEST_VERSION(0,17,91))
+        return true;
+#else
+        return false;
+#endif
+    }
+
+    return false;
 }
 
 QString KExiv2::Exiv2Version()
@@ -123,36 +179,77 @@ QString KExiv2::version()
 bool KExiv2::isReadOnly(const QString& filePath)
 {
     if (!canWriteComment(filePath))
-        return false;
+        return true;
 
     if (!canWriteExif(filePath))
-        return false;
+        return true;
 
     if (!canWriteIptc(filePath))
-        return false;
+        return true;
 
 #ifdef _XMP_SUPPORT_
     if (!canWriteXmp(filePath))
-        return false;
+        return true;
 #endif // _XMP_SUPPORT_
 
-    return true;
+    return false;
 }
 
 //-- General methods ----------------------------------------------
 
+bool KExiv2::load(const QByteArray& imgData) const
+{
+    if (imgData.isEmpty())
+        return false;
+
+    try
+    {
+        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open((Exiv2::byte*)imgData.data(), imgData.size());
+
+        d->filePath = QString();
+        image->readMetadata();
+
+        // Image comments ---------------------------------
+
+        d->imageComments = image->comment();
+
+        // Exif metadata ----------------------------------
+
+        d->exifMetadata = image->exifData();
+
+        // Iptc metadata ----------------------------------
+
+        d->iptcMetadata = image->iptcData();
+
+#ifdef _XMP_SUPPORT_
+
+        // Xmp metadata -----------------------------------
+
+        d->xmpMetadata = image->xmpData();
+
+#endif // _XMP_SUPPORT_
+
+        return true;
+    }
+    catch( Exiv2::Error &e )
+    {
+        d->printExiv2ExceptionError("Cannot load metadata using Exiv2 ", e);
+    }
+
+    return false;
+}
+
 bool KExiv2::load(const QString& filePath) const
 {
     QFileInfo finfo(filePath);
     if (filePath.isEmpty() || !finfo.isReadable())
     {
-        qDebug("File '%s' is not readable.", finfo.fileName().toAscii().constData());
+        kDebug(51003) << "File '" << finfo.fileName().toAscii().constData() << "' is not readable." << endl;
         return false;
     }
 
     try
     {
-
         Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open((const char*)
                                       (QFile::encodeName(filePath)));
 
@@ -199,12 +296,30 @@ bool KExiv2::save(const QString& filePath) const
     QFileInfo dinfo(finfo.path());
     if (!finfo.isWritable())
     {
-        qDebug("File '%s' is read-only. Metadata not saved.", finfo.fileName().toAscii().constData());
+        kDebug(51003) << "File '" << finfo.fileName().toAscii().constData() << "' is read-only. Metadata not saved." << endl;
         return false;
     }
     if (!dinfo.isWritable())
     {
-        qDebug("Dir '%s' is read-only. Metadata not saved.", dinfo.filePath().toAscii().constData());
+        kDebug(51003) << "Dir '" << dinfo.filePath().toAscii().constData() << "' is read-only. Metadata not saved." << endl;
+        return false;
+    }
+
+    // TIFF/EP Raw file based supported by Exiv2 0.18 are : DNG, NEF, PEF.
+    QString rawTiffBasedSupported("dng nef pef");
+    QString rawTiffBasedNotSupported("3fr arw cr2 dcr erf k25 kdc mos orf raw sr2 srf");
+    if (rawTiffBasedNotSupported.contains(finfo.suffix().toUpper()))
+    {
+        kDebug(51003) << "'" << dinfo.filePath().toAscii().constData() 
+                      << "' is TIFF based RAW file not yet supported. Metadata not saved." << endl;
+        return false;
+    }
+
+    if (rawTiffBasedSupported.contains(finfo.suffix().toUpper()) && !d->writeRawFiles)
+    {
+        kDebug(51003) << "'" << dinfo.filePath().toAscii().constData() 
+                      << "' is TIFF based RAW file supported but writing mode is disabled. " 
+                      << "Metadata not saved." << endl;
         return false;
     }
 
@@ -235,11 +350,12 @@ bool KExiv2::save(const QString& filePath) const
         {
             if (image->mimeType() == "image/tiff")
             {
+                Exiv2::ExifData exif = image->exifData();
+                QStringList untouchedTags;
+
                 // With tiff image we cannot overwrite whole Exif data as well, because 
                 // image data are stored in Exif container. We need to take a care about
                 // to not lost image data.
-                Exiv2::ExifData exif = image->exifData();
-                QStringList untouchedTags;
                 untouchedTags << "Exif.Image.ImageWidth";
                 untouchedTags << "Exif.Image.ImageLength";
                 untouchedTags << "Exif.Image.BitsPerSample";
@@ -293,8 +409,18 @@ bool KExiv2::save(const QString& filePath) const
 
 #endif // _XMP_SUPPORT_
 
+        // NOTE: Don't touch access and modification timestamp of file.
+        struct stat st;
+        ::stat(QFile::encodeName(filePath), &st);
+
+        struct utimbuf ut;
+        ut.modtime = st.st_mtime;
+        ut.actime  = st.st_atime;
+
         image->writeMetadata();
 
+        ::utime(QFile::encodeName(filePath), &ut);
+
         return true;
     }
     catch( Exiv2::Error &e )
@@ -331,6 +457,16 @@ QString KExiv2::getFilePath() const
     return d->filePath;
 }
 
+void KExiv2::setWriteRawFiles(bool on)
+{
+    d->writeRawFiles = on;
+}
+
+bool KExiv2::writeRawFiles() const
+{
+    return d->writeRawFiles;
+}
+
 bool KExiv2::setProgramId(bool /*on*/) const
 {
     return true;
