@@ -7,23 +7,19 @@
  * Description : Exiv2 library interface for KDE
  *               Exif manipulation methods
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
- *
- * NOTE: Do not use kdDebug() in this implementation because 
- *       it will be multithreaded. Use qDebug() instead. 
- *       See B.K.O #133026 for details.
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * ============================================================ */
 
 // Local includes.
@@ -47,7 +43,8 @@ bool KExiv2::canWriteExif(const QString& filePath)
     catch( Exiv2::Error &e )
     {
         std::string s(e.what());
-        qDebug("%s (Error #%i: %s)", "Cannot check Exif access mode using Exiv2 ", e.code(), s.c_str());
+        kDebug(51003) << "Cannot check Exif access mode using Exiv2 (Error #" 
+                      << e.code() << ": " << s.c_str() << ")" << endl;
     }
 
     return false;
@@ -106,7 +103,7 @@ QByteArray KExiv2::getExif(bool addExifHeader) const
     catch( Exiv2::Error &e )
     {
         if (!d->filePath.isEmpty())
-            qDebug ("From file %s", d->filePath.toAscii().constData());
+            kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
 
         d->printExiv2ExceptionError("Cannot get Exif data using Exiv2 ", e);
     }
@@ -134,7 +131,7 @@ bool KExiv2::setExif(const QByteArray& data) const
     catch( Exiv2::Error &e )
     {
         if (!d->filePath.isEmpty())
-            qDebug ("From file %s", d->filePath.toAscii().constData());
+            kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
 
         d->printExiv2ExceptionError("Cannot set Exif data using Exiv2 ", e);
     }
@@ -316,6 +313,8 @@ bool KExiv2::removeExifTag(const char *exifTagName, bool setProgramName) const
             d->exifMetadata.erase(it);
             return true;
         }
+
+        return true;
     }
     catch( Exiv2::Error &e )
     {
@@ -618,11 +617,16 @@ QVariant KExiv2::getExifTagVariant(const char *exifTagName, bool rationalAsListO
                 case Exiv2::unsignedLong:
                 case Exiv2::signedShort:
                 case Exiv2::signedLong:
-                    return QVariant((int)it->toLong(component));
+                    if (it->count() > component)
+                        return QVariant((int)it->toLong(component));
+                    else
+                        return QVariant(QVariant::Int);
                 case Exiv2::unsignedRational:
                 case Exiv2::signedRational:
                     if (rationalAsListOfInts)
                     {
+                        if (it->count() <= component)
+                            return QVariant(QVariant::List);
                         QList<QVariant> list;
                         list << (*it).toRational(component).first;
                         list << (*it).toRational(component).second;
@@ -630,6 +634,8 @@ QVariant KExiv2::getExifTagVariant(const char *exifTagName, bool rationalAsListO
                     }
                     else
                     {
+                        if (it->count() <= component)
+                            return QVariant(QVariant::Double);
                         // prefer double precision
                         double num = (*it).toRational(component).first;
                         double den = (*it).toRational(component).second;
@@ -743,7 +749,7 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
                 if (it != exifData.end())
                 {
                     long orientation = it->toLong();
-                    qDebug("Exif Thumbnail Orientation: %i", (int)orientation);
+                    kDebug(51003) << "Exif Thumbnail Orientation: " << (int)orientation << endl;
                     rotateExifQImage(thumbnail, (ImageOrientation)orientation);
                 }
 
@@ -815,18 +821,21 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName) const
 
     try
     {
+#if (EXIV2_TEST_VERSION(0,17,91))
+        QByteArray data;
+        QBuffer buffer(&data);
+        buffer.open(QIODevice::WriteOnly);
+        thumb.save(&buffer, "JPEG");
+        Exiv2::ExifThumb thumb(d->exifMetadata);
+        thumb.setJpegThumbnail((Exiv2::byte *)data.data(), data.size());
+#else
         KTemporaryFile thumbFile;
         thumbFile.setSuffix("KExiv2ExifThumbnail");
         thumbFile.setAutoRemove(true);
         thumbFile.open();
         thumb.save(thumbFile.fileName(), "JPEG");
-        qDebug("Thumbnail temp file: %s", thumbFile.fileName().toAscii().data());
-
+        kDebug(51003) << "Thumbnail temp file: " << thumbFile.fileName().toAscii().data() << endl;
         const std::string &fileName((const char*)(QFile::encodeName(thumbFile.fileName())));
-#if (EXIV2_TEST_VERSION(0,17,91))
-        Exiv2::ExifThumb thumb(d->exifMetadata);
-        thumb.setJpegThumbnail( fileName );
-#else
         d->exifMetadata.setJpegThumbnail( fileName );
 #endif
         return true;
