@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2004-2005 by Enrico Ros <eros.kde@email.it>             *
- *   Copyright (C) 2004-2007 by Albert Astals Cid <aacid@kde.org>          *
+ *   Copyright (C) 2004-2008 by Albert Astals Cid <aacid@kde.org>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -551,6 +551,27 @@ class OKULAR_EXPORT Document : public QObject
          */
         void unregisterView( View *view );
 
+        /**
+         * Gets the font data for the given font
+         *
+         * @since 0.8 (KDE 4.2)
+         */
+        QByteArray fontData(const FontInfo &font) const;
+
+        /**
+         * Opens a document archive.
+         *
+         * @since 0.8 (KDE 4.2)
+         */
+        bool openDocumentArchive( const QString & docFile, const KUrl & url );
+
+        /**
+         * Saves a document archive.
+         *
+         * @since 0.8 (KDE 4.2)
+         */
+        bool saveDocumentArchive( const QString &fileName );
+
     public Q_SLOTS:
         /**
          * This slot is called whenever the user changes the @p rotation of
@@ -682,7 +703,7 @@ class OKULAR_EXPORT Document : public QObject
         Q_PRIVATE_SLOT( d, void doContinueNextMatchSearch(void *pagesToNotifySet, void * match, int currentPage, int searchID, const QString & text, int caseSensitivity, bool moveViewport, const QColor & color, bool noDialogs, int donePages) )
         Q_PRIVATE_SLOT( d, void doContinuePrevMatchSearch(void *pagesToNotifySet, void * match, int currentPage, int searchID, const QString & text, int caseSensitivity, bool moveViewport, const QColor & color, bool noDialogs, int donePages) )
         Q_PRIVATE_SLOT( d, void doContinueAllDocumentSearch(void *pagesToNotifySet, void *pageMatchesMap, int currentPage, int searchID, const QString & text, int caseSensitivity, const QColor & color) )
-        Q_PRIVATE_SLOT( d, void doContinueGooglesDocumentSearch(void *pagesToNotifySet, void *pageMatchesMap, int currentPage, int searchID, const QString & text, int caseSensitivity, const QColor & color, bool matchAll) )
+        Q_PRIVATE_SLOT( d, void doContinueGooglesDocumentSearch(void *pagesToNotifySet, void *pageMatchesMap, int currentPage, int searchID, const QStringList & words, int caseSensitivity, const QColor & color, bool matchAll) )
 };
 
 
@@ -813,15 +834,19 @@ class OKULAR_EXPORT DocumentInfo : public QDomDocument
  * @short A DOM tree that describes the Table of Contents.
  *
  * The Synopsis (TOC or Table Of Contents for friends) is represented via
- * a dom tree where each node has an internal name (displayed in the listview)
+ * a dom tree where each node has an internal name (displayed in the TOC)
  * and one or more attributes.
  *
  * In the tree the tag name is the 'screen' name of the entry. A tag can have
  * attributes. Here follows the list of tag attributes with meaning:
- * - Icon: An icon to be set in the Lisview for the node
- * - Viewport: A string description of the referred viewport
- * - ViewportName: A 'named reference' to the viewport that must be converted
- *      using metaData( "NamedViewport", *viewport_name* )
+ * - Destination: A string description of the referred viewport
+ * - DestinationName: A 'named reference' to the viewport that must be converted
+ *      using metaData( "NamedViewport", viewport_name )
+ * - ExternalFileName: A document to be opened, whose destination is specified
+ *      with Destination or DestinationName
+ * - Open: a boolean saying whether its TOC branch is open or not (default: false)
+ * - URL: a URL to be open as destination; if set, no other Destination* or
+ *      ExternalFileName entry is used
  */
 class OKULAR_EXPORT DocumentSynopsis : public QDomDocument
 {
