@@ -429,7 +429,8 @@ class KDjVu::Private
 {
     public:
         Private()
-          : m_djvu_cxt( 0 ), m_djvu_document( 0 ), m_format( 0 ), m_docBookmarks( 0 )
+          : m_djvu_cxt( 0 ), m_djvu_document( 0 ), m_format( 0 ), m_docBookmarks( 0 ),
+            m_cacheEnabled( true )
         {
         }
 
@@ -442,10 +443,11 @@ class KDjVu::Private
 
         void readMetaData( int page );
 
+        int pageWithName( const QString & name );
+
         ddjvu_context_t *m_djvu_cxt;
         ddjvu_document_t *m_djvu_document;
         ddjvu_format_t *m_format;
-        unsigned int* m_formatmask;
 
         QVector<KDjVu::Page*> m_pages;
         QVector<ddjvu_page_t *> m_pages_cache;
@@ -454,8 +456,16 @@ class KDjVu::Private
 
         QHash<QString, QVariant> m_metaData;
         QDomDocument * m_docBookmarks;
+
+        QHash<QString, int> m_pageNamesCache;
+
+        bool m_cacheEnabled;
+
+        static unsigned int s_formatmask[4];
 };
 
+unsigned int KDjVu::Private::s_formatmask[4] = { 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000 };
+
 QImage KDjVu::Private::generateImageTile( ddjvu_page_t *djvupage, int& res,
     int width, int row, int xdelta, int height, int col, int ydelta )
 {
@@ -478,23 +488,16 @@ QImage KDjVu::Private::generateImageTile( ddjvu_page_t *djvupage, int& res,
     kDebug() << "pagerect:" << pagerect;
 #endif
     handle_ddjvu_messages( m_djvu_cxt, false );
-    char* imagebuffer = new char[ realwidth * realheight * 4 + 1 ];
+    QImage res_img( realwidth, realheight, QImage::Format_RGB32 );
     // the following line workarounds a rare crash in djvulibre;
     // it should be fixed with >= 3.5.21
     ddjvu_page_get_width( djvupage );
     res = ddjvu_page_render( djvupage, DDJVU_RENDER_COLOR,
-                  &pagerect, &renderrect, m_format, realwidth * 4, imagebuffer );
+                  &pagerect, &renderrect, m_format, res_img.bytesPerLine(), (char *)res_img.bits() );
 #ifdef KDJVU_DEBUG
     kDebug() << "rendering result:" << res;
 #endif
     handle_ddjvu_messages( m_djvu_cxt, false );
-    QImage res_img;
-    if ( res )
-    {
-        QImage img( (uchar*)imagebuffer, realwidth, realheight, QImage::Format_RGB32 );
-        res_img = img.copy();
-    }
-    delete [] imagebuffer;
 
     return res_img;
 }
@@ -535,14 +538,30 @@ void KDjVu::Private::fillBookmarksRecurse( QDomDocument& maindoc, QDomNode& curn
         {
             QString title = QString::fromUtf8( miniexp_to_str( miniexp_nth( 0, cur ) ) );
             QString dest = QString::fromUtf8( miniexp_to_str( miniexp_nth( 1, cur ) ) );
-            QDomElement el;
-            if ( dest.isEmpty() || ( ( dest.at( 0 ) == QLatin1Char( '#' ) ) && ( dest.remove( 0, 1 ) != title ) ) )
+            QDomElement el = maindoc.createElement( "item" );
+            el.setAttribute( "title", title );
+            if ( !dest.isEmpty() )
             {
-                el = maindoc.createElement( "item" );
-                el.setAttribute( "title", title );
-                el.setAttribute( "destination", dest );
-                curnode.appendChild( el );
+                if ( dest.at( 0 ) == QLatin1Char( '#' ) )
+                {
+                    dest.remove( 0, 1 );
+                    bool isNumber = false;
+                    dest.toInt( &isNumber );
+                    if ( isNumber )
+                    {
+                       el.setAttribute( "PageNumber", dest );
+                    }
+                    else
+                    {
+                       el.setAttribute( "PageName", dest );
+                    }
+                }
+                else
+                {
+                    el.setAttribute( "URL", dest );
+                }
             }
+            curnode.appendChild( el );
             if ( !el.isNull() && ( miniexp_length( cur ) > 2 ) )
             {
                 fillBookmarksRecurse( maindoc, el, cur, 2 );
@@ -581,6 +600,23 @@ void KDjVu::Private::readMetaData( int page )
     }
 }
 
+int KDjVu::Private::pageWithName( const QString & name )
+{
+    const QByteArray utfName = name.toUtf8();
+    const int fileNum = ddjvu_document_get_filenum( m_djvu_document );
+    ddjvu_fileinfo_t info;
+    for ( int i = 0; i < fileNum; ++i )
+    {
+        if ( DDJVU_JOB_OK != ddjvu_document_get_fileinfo( m_djvu_document, i, &info ) )
+            continue;
+        if ( info.type != 'P' )
+            continue;
+        if ( ( utfName == info.id ) || ( utfName == info.name ) || ( utfName == info.title ) )
+            return info.pageno;
+    }
+    return -1;
+}
+
 
 KDjVu::KDjVu() : d( new Private )
 {
@@ -588,18 +624,9 @@ KDjVu::KDjVu() : d( new Private )
     d->m_djvu_cxt = ddjvu_context_create( "KDjVu" );
     // creating the rendering format
 #if DDJVUAPI_VERSION >= 18
-    d->m_formatmask = new unsigned int[4];
-    d->m_formatmask[0] = 0x00ff0000;
-    d->m_formatmask[1] = 0x0000ff00;
-    d->m_formatmask[2] = 0x000000ff;
-    d->m_formatmask[3] = 0xff000000;
-    d->m_format = ddjvu_format_create( DDJVU_FORMAT_RGBMASK32, 4, d->m_formatmask );
+    d->m_format = ddjvu_format_create( DDJVU_FORMAT_RGBMASK32, 4, Private::s_formatmask );
 #else
-    d->m_formatmask = new unsigned int[3];
-    d->m_formatmask[0] = 0x00ff0000;
-    d->m_formatmask[1] = 0x0000ff00;
-    d->m_formatmask[2] = 0x000000ff;
-    d->m_format = ddjvu_format_create( DDJVU_FORMAT_RGBMASK32, 3, d->m_formatmask );
+    d->m_format = ddjvu_format_create( DDJVU_FORMAT_RGBMASK32, 3, Private::s_formatmask );
 #endif
     ddjvu_format_set_row_order( d->m_format, 1 );
     ddjvu_format_set_y_direction( d->m_format, 1 );
@@ -611,7 +638,6 @@ KDjVu::~KDjVu()
     closeFile();
 
     ddjvu_format_release( d->m_format );
-    delete [] d->m_formatmask;
     ddjvu_context_release( d->m_djvu_cxt );
 
     delete d;
@@ -714,6 +740,8 @@ void KDjVu::closeFile()
     d->mImgCache.clear();
     // clearing the old metadata
     d->m_metaData.clear();
+    // cleaing the page names mapping
+    d->m_pageNamesCache.clear();
     // releasing the old document
     if ( d->m_djvu_document )
         ddjvu_document_release( d->m_djvu_document );
@@ -722,8 +750,8 @@ void KDjVu::closeFile()
 
 QVariant KDjVu::metaData( const QString & key ) const
 {
-    QHash<QString, QVariant>::ConstIterator it = d->m_metaData.find( key );
-    return it != d->m_metaData.end() ? it.value() : QVariant();
+    QHash<QString, QVariant>::ConstIterator it = d->m_metaData.constFind( key );
+    return it != d->m_metaData.constEnd() ? it.value() : QVariant();
 }
 
 const QDomDocument * KDjVu::documentBookmarks() const
@@ -856,6 +884,8 @@ const QVector<KDjVu::Page*> &KDjVu::pages() const
 
 QImage KDjVu::image( int page, int width, int height, int rotation )
 {
+    if ( d->m_cacheEnabled )
+    {
     bool found = false;
     QList<ImageCacheItem*>::Iterator it = d->mImgCache.begin(), itEnd = d->mImgCache.end();
     for ( ; ( it != itEnd ) && !found; ++it )
@@ -877,6 +907,7 @@ QImage KDjVu::image( int page, int width, int height, int rotation )
 
         return cur2->img;
     }
+    }
 
     if ( !d->m_pages_cache.at( page ) )
     {
@@ -904,7 +935,7 @@ QImage KDjVu::image( int page, int width, int height, int rotation )
     int xparts = width / xdelta + 1;
     int yparts = height / ydelta + 1;
 
-    QImage newimg( width, height, QImage::Format_RGB32 );
+    QImage newimg;
 
     int res = 10000;
     if ( ( xparts == 1 ) && ( yparts == 1 ) )
@@ -917,6 +948,7 @@ QImage KDjVu::image( int page, int width, int height, int rotation )
     {
         // more than one part -- need to render piece-by-piece and to compose
         // the results
+        newimg = QImage( width, height, QImage::Format_RGB32 );
         QPainter p;
         p.begin( &newimg );
         int parts = xparts * yparts;
@@ -936,7 +968,7 @@ QImage KDjVu::image( int page, int width, int height, int rotation )
         p.end();
     }
 
-    if ( res )
+    if ( res && d->m_cacheEnabled )
     {
         // delete all the cached pixmaps for the current page with a size that
         // differs no more than 35% of the new pixmap size
@@ -1075,3 +1107,34 @@ QList<KDjVu::TextEntity> KDjVu::textEntities( int page, const QString & granular
 
     return ret;
 }
+
+void KDjVu::setCacheEnabled( bool enable )
+{
+    if ( enable == d->m_cacheEnabled )
+        return;
+
+    d->m_cacheEnabled = enable;
+    if ( !d->m_cacheEnabled )
+    {
+        qDeleteAll( d->mImgCache );
+        d->mImgCache.clear();
+    }
+}
+
+bool KDjVu::isCacheEnabled() const
+{
+    return d->m_cacheEnabled;
+}
+
+int KDjVu::pageNumber( const QString & name ) const
+{
+    if ( !d->m_djvu_document )
+        return -1;
+
+    QHash< QString, int >::iterator it = d->m_pageNamesCache.find( name );
+    if ( it == d->m_pageNamesCache.end() )
+    {
+        it = d->m_pageNamesCache.insert( name, d->pageWithName( name ) );
+    }
+    return it.value();
+}
