@@ -33,6 +33,7 @@
 #include <okular/core/action.h>
 #include <okular/core/page.h>
 #include <okular/core/annotations.h>
+#include <okular/core/movie.h>
 #include <okular/core/pagetransition.h>
 #include <okular/core/sound.h>
 #include <okular/core/sourcereference.h>
@@ -45,6 +46,10 @@
 #include "formfields.h"
 #include "popplerembeddedfile.h"
 
+#ifdef HAVE_POPPLER_0_9
+Q_DECLARE_METATYPE(Poppler::FontInfo)
+#endif
+
 static const int PDFDebug = 4710;
 
 class PDFOptionsPage : public QWidget
@@ -134,7 +139,19 @@ Okular::Sound* createSoundFromPopplerSound( const Poppler::SoundObject *popplerS
     return sound;
 }
 
-static Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
+#ifdef HAVE_POPPLER_0_9
+Okular::Movie* createMovieFromPopplerMovie( const Poppler::MovieObject *popplerMovie )
+{
+    Okular::Movie *movie = new Okular::Movie( popplerMovie->url() );
+    movie->setSize( popplerMovie->size() );
+    movie->setRotation( (Okular::Rotation)( popplerMovie->rotation() / 90 ) );
+    movie->setShowControls( popplerMovie->showControls() );
+    movie->setPlayMode( (Okular::Movie::PlayMode)popplerMovie->playMode() );
+    return movie;
+}
+#endif
+
+Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
 {
 	Okular::Action *link = 0;
 	const Poppler::LinkGoto *popplerLinkGoto;
@@ -241,7 +258,7 @@ static KAboutData createAboutData()
          "okular_poppler",
          "okular_poppler",
          ki18n( "PDF Backend" ),
-         "0.1.1",
+         "0.2",
          ki18n( "A PDF file renderer" ),
          KAboutData::License_GPL,
          ki18n( "© 2005-2008 Albert Astals Cid" )
@@ -260,6 +277,8 @@ PDFGenerator::PDFGenerator( QObject *parent, const QVariantList &args )
     setFeature( TextExtraction );
     setFeature( FontInfo );
     setFeature( PrintPostscript );
+    if ( Okular::FilePrinter::ps2pdfAvailable() )
+        setFeature( PrintToFile );
     setFeature( ReadRawData );
     // generate the pixmapGeneratorThread
     generatorThread = new PDFPixmapGeneratorThread( this );
@@ -629,6 +648,13 @@ Okular::FontInfo::List PDFGenerator::fontsForPage( int page )
         of.setType( convertPopplerFontInfoTypeToOkularFontInfoType( font.type() ) );
         of.setEmbedType( embedTypeForPopplerFontInfo( font) );
         of.setFile( font.file() );
+#ifdef HAVE_POPPLER_0_9
+        of.setCanBeExtracted( of.embedType() != Okular::FontInfo::NotEmbedded );
+        
+        QVariant nativeId;
+        nativeId.setValue( font );
+        of.setNativeId( nativeId );
+#endif
 
         list.append( of );
     }
@@ -804,12 +830,25 @@ Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
     return tp;
 }
 
+void PDFGenerator::requestFontData(const Okular::FontInfo &font, QByteArray *data)
+{
+#ifdef HAVE_POPPLER_0_9
+    Poppler::FontInfo fi = font.nativeId().value<Poppler::FontInfo>();
+    *data = pdfdoc->fontData(fi);
+#else
+    Q_UNUSED( font )
+    Q_UNUSED( data )
+#endif
+}
+
 bool PDFGenerator::print( QPrinter& printer )
 {
     // Get the real page size to pass to the ps generator
     QPrinter dummy( QPrinter::PrinterResolution );
     dummy.setFullPage( true );
     dummy.setOrientation( printer.orientation() );
+    dummy.setPageSize( printer.pageSize() );
+    dummy.setPaperSize( printer.paperSize( QPrinter::Millimeter ), QPrinter::Millimeter );
     int width = dummy.width();
     int height = dummy.height();
 
@@ -1210,6 +1249,7 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, QDomNode * parentDest
             item.setAttribute( "Viewport", vp.toString() );
         }
         if (!e.attribute("Open").isNull()) item.setAttribute("Open", e.attribute("Open"));
+        if (!e.attribute("DestinationURI").isNull()) item.setAttribute("URL", e.attribute("DestinationURI"));
 
         // descend recursively and advance to the next node
         if ( e.hasChildNodes() ) addSynopsisChildren( &n, &	item );
@@ -1422,8 +1462,8 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
         if ( line.startsWith( QLatin1Char( 'l' ) ) && linere.exactMatch( line ) )
         {
             int id = linere.cap( 1 ).toInt();
-            QHash<int, pdfsyncpoint>::const_iterator it = points.find( id );
-            if ( it == points.end() )
+            QHash<int, pdfsyncpoint>::const_iterator it = points.constFind( id );
+            if ( it == points.constEnd() )
             {
                 pdfsyncpoint pt;
                 pt.x = 0;
