@@ -176,8 +176,8 @@ static QString compressedMimeFor( const QString& mime_to_check )
         compressedMimeMap[ QString::fromLatin1( "image/x-bzeps" ) ] =
             QString::fromLatin1( "application/x-bzip" );
     }
-    QHash< QString, QString >::const_iterator it = compressedMimeMap.find( mime_to_check );
-    if ( it != compressedMimeMap.end() )
+    QHash< QString, QString >::const_iterator it = compressedMimeMap.constFind( mime_to_check );
+    if ( it != compressedMimeMap.constEnd() )
         return it.value();
 
     return QString();
@@ -513,6 +513,11 @@ m_cliPresentation(false), m_generatorGuiClient(0), m_keeper( 0 )
     m_exportAsMenu->addAction( m_exportAsText );
     m_exportAs->setEnabled( false );
     m_exportAsText->setEnabled( false );
+    m_exportAsDocArchive = actionForExportFormat( Okular::ExportFormat(
+            i18nc( "A document format, Okular-specific", "Document Archive" ),
+            KMimeType::mimeType( "application/vnd.kde.okular-archive" ) ), m_exportAsMenu );
+    m_exportAsMenu->addAction( m_exportAsDocArchive );
+    m_exportAsDocArchive->setEnabled( false );
 
     m_aboutBackend = ac->addAction("help_about_backend");
     m_aboutBackend->setText(i18n("About Backend"));
@@ -533,6 +538,19 @@ m_cliPresentation(false), m_generatorGuiClient(0), m_keeper( 0 )
     closeFindBar->setShortcut( QKeySequence( Qt::Key_Escape ) );
     widget()->addAction(closeFindBar);
 
+    KToggleAction *blackscreenAction = new KToggleAction( i18n( "Switch Blackscreen Mode" ), ac );
+    ac->addAction( "switch_blackscreen_mode", blackscreenAction );
+    blackscreenAction->setShortcut( QKeySequence( Qt::Key_B ) );
+    blackscreenAction->setIcon( KIcon( "view-presentation" ) );
+
+    KToggleAction *drawingAction = new KToggleAction( i18n( "Toggle Drawing Mode" ), ac );
+    ac->addAction( "presentation_drawing_mode", drawingAction );
+    drawingAction->setIcon( KIcon( "draw-freehand" ) );
+
+    KAction *eraseDrawingAction = new KAction( i18n( "Erase Drawings" ), ac );
+    ac->addAction( "presentation_erase_drawings", eraseDrawingAction );
+    eraseDrawingAction->setIcon( KIcon( "draw-eraser" ) );
+
     // document watcher and reloader
     m_watcher = new KDirWatch( this );
     connect( m_watcher, SIGNAL( dirty( const QString& ) ), this, SLOT( slotFileDirty( const QString& ) ) );
@@ -689,16 +707,20 @@ void Part::loadCancelled(const QString &reason)
 
 void Part::setWindowTitleFromDocument()
 {
-    // if the document have a 'DocumentTitle' flag set (and it is not empty), set it as title
-    QString title = m_document->metaData( "DocumentTitle" ).toString();
-    if ( !title.isEmpty() && !title.trimmed().isEmpty() )
+    // If 'DocumentTitle' should be used, check if the document has one. If
+    // either case is false, use the file name.
+    QString title = realUrl().fileName();
+
+    if ( Okular::Settings::displayDocumentTitle() )
     {
-        emit setWindowCaption( title );
-    }
-    else
-    {
-        emit setWindowCaption( realUrl().fileName() );
+        const QString docTitle = m_document->metaData( "DocumentTitle" ).toString();
+        if ( !docTitle.isEmpty() && !docTitle.trimmed().isEmpty() )
+        {
+             title = docTitle;
+        }
     }
+
+    emit setWindowCaption( title );
 }
 
 void Part::slotGeneratorPreferences( )
@@ -846,7 +868,10 @@ bool Part::openFile()
     bool ok = false;
     if ( uncompressOk )
     {
-        ok = m_document->openDocument( fileNameToOpen, url(), mime );
+        if ( mime->is( "application/vnd.kde.okular-archive" ) )
+            ok = m_document->openDocumentArchive( fileNameToOpen, url() );
+        else
+            ok = m_document->openDocument( fileNameToOpen, url(), mime );
     }
     bool canSearch = m_document->supportsSearching();
 
@@ -884,11 +909,12 @@ bool Part::openFile()
 #endif
     }
     m_exportAsText->setEnabled( ok && m_document->canExportToText() );
+    m_exportAsDocArchive->setEnabled( ok );
     m_exportAs->setEnabled( ok );
 
     // update viewing actions
     updateViewActions();
-    
+
     m_fileWasRemoved = false;
 
     if ( !ok )
@@ -972,11 +998,12 @@ bool Part::closeUrl()
     m_showEmbeddedFiles->setEnabled( false );
     m_exportAs->setEnabled( false );
     m_exportAsText->setEnabled( false );
+    m_exportAsDocArchive->setEnabled( false );
     m_exportFormats.clear();
     QMenu *menu = m_exportAs->menu();
     QList<QAction*> acts = menu->actions();
     int num = acts.count();
-    for ( int i = 1; i < num; ++i )
+    for ( int i = 2; i < num; ++i )
     {
         menu->removeAction( acts.at(i) );
         delete acts.at(i);
@@ -1680,8 +1707,7 @@ void Part::slotShowPresentation()
 {
     if ( !m_presentationWidget )
     {
-        m_presentationWidget = new PresentationWidget( widget(), m_document );
-        m_presentationWidget->setupActions( actionCollection() );
+        m_presentationWidget = new PresentationWidget( widget(), m_document, actionCollection() );
     }
 }
 
@@ -1698,12 +1724,21 @@ void Part::slotTogglePresentation()
     if ( m_document->isOpened() )
     {
         if ( !m_presentationWidget )
-            m_presentationWidget = new PresentationWidget( widget(), m_document );
+            m_presentationWidget = new PresentationWidget( widget(), m_document, actionCollection() );
         else delete (PresentationWidget*) m_presentationWidget;
     }
 }
 
 
+void Part::reload()
+{
+    if ( m_document->isOpened() )
+    {
+        slotReload();
+    }
+}
+
+
 void Part::slotAboutBackend()
 {
     const KComponentData *data = m_document->componentData();
@@ -1722,11 +1757,35 @@ void Part::slotExportAs(QAction * act)
     if ( ( id < 0 ) || ( id >= acts.count() ) )
         return;
 
-    QString filter = id == 0 ? "text/plain" : m_exportFormats.at( id - 1 ).mimeType()->name();
+    QString filter;
+    switch ( id )
+    {
+        case 0:
+            filter = "text/plain";
+            break;
+        case 1:
+            filter = "application/vnd.kde.okular-archive";
+            break;
+        default:
+            filter = m_exportFormats.at( id - 2 ).mimeType()->name();
+            break;
+    }
     QString fileName = KFileDialog::getSaveFileName( url().isLocalFile() ? url().directory() : QString(), filter, widget() );
     if ( !fileName.isEmpty() )
     {
-        bool saved = id == 0 ? m_document->exportToText( fileName ) : m_document->exportTo( fileName, m_exportFormats.at( id - 1 ) );
+        bool saved = false;
+        switch ( id )
+        {
+            case 0:
+                saved = m_document->exportToText( fileName );
+                break;
+            case 1:
+                saved = m_document->saveDocumentArchive( fileName );
+                break;
+            default:
+                saved = m_document->exportTo( fileName, m_exportFormats.at( id - 2 ) );
+                break;
+        }
         if ( !saved )
             KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", fileName ) );
     }
