@@ -10,6 +10,10 @@
 #include "presentationwidget.h"
 
 // qt/kde includes
+#include <QtDBus/QDBusConnection>
+#include <QtDBus/QDBusMessage>
+#include <QtDBus/QDBusReply>
+
 #include <qevent.h>
 #include <qfontmetrics.h>
 #include <kicon.h>
@@ -44,10 +48,13 @@
 #include "annotationtools.h"
 #include "pagepainter.h"
 #include "presentationsearchbar.h"
+#include "videowidget.h"
 #include "core/action.h"
+#include "core/annotations.h"
 #include "core/audioplayer.h"
 #include "core/document.h"
 #include "core/generator.h"
+#include "core/movie.h"
 #include "core/page.h"
 #include "settings.h"
 
@@ -60,6 +67,11 @@
 // transition effect to the next frame
 struct PresentationFrame
 {
+    ~PresentationFrame()
+    {
+        qDeleteAll( videoWidgets );
+    }
+
     void recalcGeometry( int width, int height, float screenRatio )
     {
         // calculate frame geometry keeping constant aspect ratio
@@ -73,10 +85,18 @@ struct PresentationFrame
         geometry.setRect( ( width - pageWidth ) / 2,
                           ( height - pageHeight ) / 2,
                           pageWidth, pageHeight );
+        Q_FOREACH ( VideoWidget *vw, videoWidgets )
+        {
+            const Okular::NormalizedRect r = vw->normGeometry();
+            QRect vwgeom = r.geometry( geometry.width(), geometry.height() );
+            vw->resize( vwgeom.size() );
+            vw->move( geometry.topLeft() + vwgeom.topLeft() );
+        }
     }
 
     const Okular::Page * page;
     QRect geometry;
+    QHash< Okular::Movie *, VideoWidget * > videoWidgets;
 };
 
 
@@ -104,11 +124,11 @@ class PresentationToolBar : public QToolBar
 };
 
 
-PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc )
+PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc, KActionCollection * collection )
     : QWidget( 0 /* must be null, to have an independent widget */, Qt::FramelessWindowHint ),
-    m_pressedLink( 0 ), m_handCursor( false ), m_drawingEngine( 0 ), m_document( doc ),
-    m_frameIndex( -1 ), m_topBar( 0 ), m_pagesEdit( 0 ), m_searchBar( 0 ),
-    m_screenSelect( 0 ), m_blockNotifications( false )
+    m_pressedLink( 0 ), m_handCursor( false ), m_drawingEngine( 0 ), m_screenSaverCookie( -1 ),
+    m_document( doc ), m_frameIndex( -1 ), m_topBar( 0 ), m_pagesEdit( 0 ), m_searchBar( 0 ),
+    m_screenSelect( 0 ), m_blockNotifications( false ), m_inBlackScreenMode( false )
 {
     Q_UNUSED( parent )
     setAttribute( Qt::WA_DeleteOnClose );
@@ -145,11 +165,14 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     connect( m_pagesEdit, SIGNAL( returnPressed() ), this, SLOT( slotPageChanged() ) );
     m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "go-previous" : "go-next" ), i18n( "Next Page" ), this, SLOT( slotNextPage() ) );
     m_topBar->addSeparator();
-    QAction * drawingAct = m_topBar->addAction( KIcon( "draw-freehand" ), i18n( "Toggle Drawing Mode" ) );
-    drawingAct->setCheckable( true );
-    connect( drawingAct, SIGNAL( toggled( bool ) ), this, SLOT( togglePencilMode( bool ) ) );
-    QAction * eraseDrawingAct = m_topBar->addAction( KIcon( "draw-eraser" ), i18n( "Erase Drawings" ) );
-    connect( eraseDrawingAct, SIGNAL( triggered() ), this, SLOT( clearDrawings() ) );
+    QAction *drawingAct = collection->action( "presentation_drawing_mode" );
+    connect( drawingAct, SIGNAL( toggled( bool ) ), SLOT( togglePencilMode( bool ) ) );
+    m_topBar->addAction( drawingAct );
+    addAction( drawingAct );
+    QAction *eraseDrawingAct = collection->action( "presentation_erase_drawings" );
+    connect( eraseDrawingAct, SIGNAL( triggered() ), SLOT( clearDrawings() ) );
+    m_topBar->addAction( eraseDrawingAct );
+    addAction( eraseDrawingAct );
     QDesktopWidget *desktop = QApplication::desktop();
     if ( desktop->numScreens() > 1 )
     {
@@ -201,11 +224,19 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
         setCursor( QCursor( Qt::BlankCursor ) );
     }
 
+    setupActions( collection );
+
+    // inhibit the screen saver
+    inhibitScreenSaver();
+
     QTimer::singleShot( 0, this, SLOT( slotDelayedEvents() ) );
 }
 
 PresentationWidget::~PresentationWidget()
 {
+    // allow the screen saver again
+    allowScreenSaver();
+
     // stop the audio playbacks
     Okular::AudioPlayer::instance()->stopPlaybacks();
 
@@ -218,6 +249,10 @@ PresentationWidget::~PresentationWidget()
     // remove this widget from document observer
     m_document->removeObserver( this );
 
+    QAction *drawingAct = m_ac->action( "presentation_drawing_mode" );
+    disconnect( drawingAct, 0, this, 0 );
+    if ( drawingAct->isChecked() )
+        drawingAct->toggle();
     m_document->removePageAnnotations( m_document->viewport().pageNumber, m_currentPageDrawings );
     delete m_drawingEngine;
 
@@ -250,6 +285,19 @@ void PresentationWidget::notifySetup( const QVector< Okular::Page * > & pageSet,
     {
         PresentationFrame * frame = new PresentationFrame();
         frame->page = *setIt;
+        const QLinkedList< Okular::Annotation * > annotations = (*setIt)->annotations();
+        QLinkedList< Okular::Annotation * >::const_iterator aIt = annotations.begin(), aEnd = annotations.end();
+        for ( ; aIt != aEnd; ++aIt )
+        {
+            Okular::Annotation * a = *aIt;
+            if ( a->subType() == Okular::Annotation::AMovie )
+            {
+                Okular::MovieAnnotation * movieAnn = static_cast< Okular::MovieAnnotation * >( a );
+                VideoWidget * vw = new VideoWidget( movieAnn, m_document, this );
+                frame->videoWidgets.insert( movieAnn->movie(), vw );
+                vw->hide();
+            }
+        }
         frame->recalcGeometry( m_width, m_height, screenRatio );
         // add the frame to the vector
         m_frames.push_back( frame );
@@ -314,6 +362,10 @@ void PresentationWidget::setupActions( KActionCollection * collection )
     addAction( m_ac->action( "go_next" ) );
     addAction( m_ac->action( "first_page" ) );
     addAction( m_ac->action( "last_page" ) );
+
+    QAction *action = m_ac->action( "switch_blackscreen_mode" );
+    connect( action, SIGNAL( toggled( bool ) ), SLOT( toggleBlackScreenMode( bool ) ) );
+    addAction( action );
 }
 
 
@@ -364,6 +416,8 @@ void PresentationWidget::keyPressEvent( QKeyEvent * e )
 
 void PresentationWidget::wheelEvent( QWheelEvent * e )
 {
+    if (m_width == -1) return;
+
     // performance note: don't remove the clipping
     int div = e->delta() / 120;
     if ( div > 0 )
@@ -384,6 +438,8 @@ void PresentationWidget::wheelEvent( QWheelEvent * e )
 
 void PresentationWidget::mousePressEvent( QMouseEvent * e )
 {
+    if (m_width == -1) return;
+
     if ( m_drawingEngine )
     {
         QRect r = routeMouseDrawingEvent( e );
@@ -491,6 +547,13 @@ void PresentationWidget::mouseMoveEvent( QMouseEvent * e )
 
 void PresentationWidget::paintEvent( QPaintEvent * pe )
 {
+    if ( m_inBlackScreenMode )
+    {
+        QPainter painter( this );
+        painter.fillRect( pe->rect(), Qt::black );
+        return;
+    }
+
     if (m_width == -1)
     {
         m_width = width();
@@ -561,6 +624,17 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
     }
     painter.end();
 }
+
+void PresentationWidget::resizeEvent( QResizeEvent *re )
+{
+    // kDebug() << re->oldSize() << "=>" << re->size();
+    if ( re->oldSize() == QSize( -1, -1 ) )
+        return;
+
+    m_screen = QApplication::desktop()->screenNumber( this );
+
+    applyNewScreenSize( re->oldSize() );
+}
 // </widget events>
 
 
@@ -636,6 +710,7 @@ void PresentationWidget::changePage( int newPage )
     if ( m_frameIndex == newPage )
         return;
 
+    const int oldIndex = m_frameIndex;
     // check if pixmap exists or else request it
     m_frameIndex = newPage;
     PresentationFrame * frame = m_frames[ m_frameIndex ];
@@ -677,6 +752,25 @@ void PresentationWidget::changePage( int newPage )
             m_document->processAction( m_document->page( m_frameIndex )->pageAction( Okular::Page::Opening ) );
 
     }
+
+    if ( oldIndex != m_frameIndex )
+    {
+        if ( oldIndex != -1 )
+        {
+            Q_FOREACH ( VideoWidget *vw, m_frames[ oldIndex ]->videoWidgets )
+            {
+                vw->stop();
+                vw->hide();
+            }
+        }
+
+        Q_FOREACH ( VideoWidget *vw, m_frames[ m_frameIndex ]->videoWidgets )
+        {
+            vw->show();
+            vw->raise();
+        }
+
+    }
 }
 
 void PresentationWidget::generatePage( bool disableTransition )
@@ -1213,6 +1307,13 @@ void PresentationWidget::chooseScreen( QAction *act )
     setScreen( newScreen );
 }
 
+void PresentationWidget::toggleBlackScreenMode( bool )
+{
+    m_inBlackScreenMode = !m_inBlackScreenMode;
+
+    update();
+}
+
 void PresentationWidget::setScreen( int newScreen )
 {
     const QRect screenGeom = QApplication::desktop()->screenGeometry( newScreen );
@@ -1221,6 +1322,11 @@ void PresentationWidget::setScreen( int newScreen )
     m_screen = newScreen;
     setGeometry( screenGeom );
 
+    applyNewScreenSize( oldSize );
+}
+
+void PresentationWidget::applyNewScreenSize( const QSize & oldSize )
+{
     repositionContent();
 
     // if by chance the new screen has the same resolution of the previous,
@@ -1232,23 +1338,48 @@ void PresentationWidget::setScreen( int newScreen )
     m_height = height();
 
     // update the frames
-    QVector< PresentationFrame * >::const_iterator fIt = m_frames.begin(), fEnd = m_frames.end();
+    QVector< PresentationFrame * >::const_iterator fIt = m_frames.constBegin(), fEnd = m_frames.constEnd();
     const float screenRatio = (float)m_height / (float)m_width;
     for ( ; fIt != fEnd; ++fIt )
     {
         (*fIt)->recalcGeometry( m_width, m_height, screenRatio );
     }
 
-    // uglyness alarm!
+    if ( m_frameIndex != -1 )
+    {
+    // ugliness alarm!
     const_cast< Okular::Page * >( m_frames[ m_frameIndex ]->page )->deletePixmap( PRESENTATION_ID );
     // force the regeneration of the pixmap
     m_lastRenderedPixmap = QPixmap();
     m_blockNotifications = true;
     requestPixmaps();
     m_blockNotifications = false;
+    }
     generatePage( true /* no transitions */ );
 }
 
+void PresentationWidget::inhibitScreenSaver()
+{
+    QDBusMessage message = QDBusMessage::createMethodCall( "org.freedesktop.ScreenSaver", "/ScreenSaver",
+                                                           "org.freedesktop.ScreenSaver", "Inhibit" );
+    message << QString( "Okular" );
+    message << i18nc( "Reason for inhibiting the screensaver activation, when the presentation mode is active", "Giving a presentation" );
+
+    QDBusReply<uint> reply = QDBusConnection::sessionBus().call( message );
+    if ( reply.isValid() )
+        m_screenSaverCookie = reply.value();
+}
+
+void PresentationWidget::allowScreenSaver()
+{
+    if ( m_screenSaverCookie != -1 )
+    {
+        QDBusMessage message = QDBusMessage::createMethodCall( "org.freedesktop.ScreenSaver", "/ScreenSaver",
+                                                               "org.freedesktop.ScreenSaver", "UnInhibit" );
+        message << (uint)m_screenSaverCookie;
+        QDBusConnection::sessionBus().send( message );
+    }
+}
 
 void PresentationWidget::slotFind()
 {
