@@ -379,9 +379,7 @@ Sidebar::Sidebar( QWidget *parent )
     d->list->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
     d->list->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
     d->list->setContextMenuPolicy( Qt::CustomContextMenu );
-    QPalette pal = d->list->palette();
-    pal.setBrush( QPalette::Base, pal.brush( QPalette::Window ) );
-    d->list->setPalette( pal );
+    d->list->viewport()->setAutoFillBackground( false );
 
     d->splitter = new QSplitter( this );
     mainlay->addWidget( d->splitter );
@@ -390,7 +388,7 @@ Sidebar::Sidebar( QWidget *parent )
 
     d->sideContainer = new QWidget( d->splitter );
     d->sideContainer->setMinimumWidth( 90 );
-    d->sideContainer->setMaximumWidth( 300 );
+    d->sideContainer->setMaximumWidth( 600 );
     d->vlay = new QVBoxLayout( d->sideContainer );
     d->vlay->setMargin( 0 );
 
